<?php

class visits_counter {
    
    private $online_counter_table;
    private $overall_counter_table;
    private $options_table;
    private $time_range;
    
    private $section_id;
    private $section_title;
    private $section_callback;
    private $section_page;
    
    private $online_offset_id;
    private $online_offset_title;
    private $online_offset_callback;
    private $online_offset_value;
    private $online_offset_default_value;
    
    private $overall_offset_id;
    private $overall_offset_title;
    private $overall_offset_callback;
    private $overall_offset_value;
    private $overall_offset_default_value;
    
    private $startTime_id;
    private $startTime_title;
    private $startTime_callback;
    private $startTime_value;
    private $startTime_default_value;
    
    private $startTimeFormat_id;
    private $startTimeFormat_title;
    private $startTimeFormat_callback;
    private $startTimeFormat_value;
    private $startTimeFormat_default_value;
    
    private $disconnectTime_id;
    private $disconnectTime_title;
    private $disconnectTime_callback;
    private $disconnectTime_value;
    private $disconnectTime_default_value;
    
    private $countBots_id;
    private $countBots_title;
    private $countBots_callback;
    private $countBots_value;
    private $countBots_default_value;
    
    private $options_page_name;
    /*
     * Class constructor
     * @params: $time = 120 - time range within user is online on page [in seconds]
     * @return: WP Error if time is less than 0 or none if everything is ok
     */
    public function __construct($time = 120){
        global $wpdb;
        $this->online_counter_table = $wpdb->prefix."vs_current_online_users";
        $this->overall_counter_table = $wpdb->prefix."vs_overall_counter";
        $this->options_table = $wpdb->prefix."vs_options";
        
        $this->online_offset_id = 'vs_online_offset';
        $this->overall_offset_id = 'vs_overall_offset';
        $this->startTime_id = 'vs_startUnixTimeStamp';
        $this->startTimeFormat_id = 'vs_startTimeFormat';
        $this->disconnectTime_id = 'vs_disconnectTime';
        $this->countBots_id = 'vs_countBots';
        
        $this->section_id = 'vs';
        
        $this->options_page_name = 'general';
        
        $this->online_offset_default_value = 0;
        $this->overall_offset_default_value = 0;
        $this->startTime_default_value = time();
        $this->startTimeFormat_default_value = "Y-m-d H:s";
        $this->disconnectTime_default_value = 300; // in seconds
        $this->countBots_default_value = 0; //false
        
        if ($time < 0)
            return new WP_Error('wrong_time', "I can't check users from future. Parameter time range can't be less than 0 in visits counter plugin.");
        else
            $this->time_range = $time;
        
        load_plugin_textdomain('visits_counter', false, basename( dirname( __FILE__ ) ) . '/languages/');
    }
   

    /*
     * Installation function, it creates requires db tables if no exists
     * @params: none
     * @return: none
     */
    public function vc_installation(){
        $this->create_tables();
        $this->create_options();
    }
    
    /*
     * Admin options initialization
     * @params: none
     * @return: none
     */
    public function vc_adminInit(){
        register_setting($this->options_page_name, $this->online_offset_id, array($this, 'prepare_offset_value'));
        register_setting($this->options_page_name, $this->overall_offset_id, array($this, 'prepare_offset_value'));
        register_setting($this->options_page_name, $this->startTime_id, array($this, 'prepare_new_startTime'));
        register_setting($this->options_page_name, $this->startTimeFormat_id, array($this, 'prepare_startTimeFormat_value'));
        register_setting($this->options_page_name, $this->disconnectTime_id, array($this, 'prepare_disconnectTime_value'));
        register_setting($this->options_page_name, $this->countBots_id, array($this, 'prepare_countBots_value'));
        $this->create_options_form();
    }
    
    /*
     * Method which create new tables in wp db (if it is neccessary)
     * @params: none
     * @return: none
     */
    private function create_tables(){
        global $wpdb;
        if ($wpdb->get_var("show tables like '$this->online_counter_table'") != $this->online_counter_table){
            $wpdb->query("CREATE TABLE IF NOT EXISTS `$this->online_counter_table` (
                                    `IP` varchar(15) NOT NULL COMMENT 'IP of user which enters page',
                                    `time` varchar(255) NOT NULL COMMENT 'time when user enters page (unix timestamp)',
                                PRIMARY KEY (`IP`)
                                ) ENGINE=MyISAM DEFAULT CHARSET=utf8;");
        }
        if ($wpdb->get_var("show tables like '$this->overall_counter_table'") != $this->overall_counter_table){
            $wpdb->query("CREATE TABLE IF NOT EXISTS `$this->overall_counter_table`(
                                    `id` int NOT NULL AUTO_INCREMENT,
                                    `visits` int NULL,
                                PRIMARY KEY (`id`)
                                ) ENGINE=MyISAM DEFAULT CHARSET=utf8;");
            $wpdb->insert($this->overall_counter_table, array('id'=>1, 'visits'=>0));
        }
    }
    
    /*
     * Method which create options records in wp_options table
     * @param: none;
     * @return: none
     */
    private function create_options(){
        add_option($this->online_offset_id, $this->online_offset_default_value, null, false);
        add_option($this->overall_offset_id, $this->overall_offset_default_value, null, false);
        add_option($this->startTime_id,$this->startTime_default_value, null, false);
        add_option($this->startTimeFormat_id, $this->startTimeFormat_default_value, null, false);
        add_option($this->disconnectTime_id, $this->disconnectTime_default_value, null, false);
        add_option($this->countBots_id, $this->countBots_default_value, null, false);
    }
    
    /*
     * Method which create options form in wp-admin panel
     * @params: none
     * @return: none
     */
    private function create_options_form(){
        
        $this->section_title = '';
        $this->section_callback = array($this, 'draw_section');
        $this->section_page = 'general';
        
        //$this->online_offset_id = 'vs_onID';
        $this->online_offset_title = __('Online Users Offset', 'visits_counter');
        $this->online_offset_callback = array($this,'draw_online_offset_field');
        
        //$this->overall_offset_id = 'vs_ovID';
        $this->overall_offset_title = __('Overall Users Offset', 'visits_counter');
        $this->overall_offset_callback = array($this,'draw_overall_offset_field');;
        
        //$this->startTime_id = 'vs_onID';
        $this->startTime_title = __('Overall counter start time', 'visits_counter');
        $this->startTime_callback = array($this,'draw_startTime_field');;
        
        $this->startTimeFormat_title = __('Start Time display format', 'visits_counter');
        $this->startTimeFormat_callback = array($this, 'draw_startTimeFormat_field');
        
        $this->disconnectTime_title = __('Disconnect Time [s]', 'visits_counter');
        $this->disconnectTime_callback = array($this, 'draw_disconnectTime_field');
        
        $this->countBots_title = __('Count Bots (like GoogleBot)', 'visits_counter');
        $this->countBots_callback = array($this, 'draw_countBots_field');
        
        add_settings_section($this->section_id, $this->section_title, $this->section_callback, $this->section_page);
        add_settings_field($this->online_offset_id, $this->online_offset_title, $this->online_offset_callback, $this->section_page, $this->section_id);
        add_settings_field($this->overall_offset_id, $this->overall_offset_title, $this->overall_offset_callback, $this->section_page, $this->section_id);
        add_settings_field($this->startTime_id, $this->startTime_title, $this->startTime_callback, $this->section_page, $this->section_id);
        add_settings_field($this->startTimeFormat_id, $this->startTimeFormat_title, $this->startTimeFormat_callback, $this->section_page, $this->section_id);
        add_settings_field($this->disconnectTime_id, $this->disconnectTime_title, $this->disconnectTime_callback, $this->section_page, $this->section_id);
        add_settings_field($this->countBots_id, $this->countBots_title, $this->countBots_callback, $this->section_page, $this->section_id);
    }
    
    /*
     * Callback method for setting section
     * @params: none
     * @return: none
     */
    public function draw_section(){
       echo '<h3>'.__('Visitors counter settings', 'visits_counter').'</h3>';
    }
    
    /*
     * Callback function for online offset setting field
     * @params: none
     * @return: none
     */
    public function draw_online_offset_field(){
        $option_value = get_option($this->online_offset_id);
        echo "<input type=\"text\" id=\"$this->online_offset_id\" name=\"$this->online_offset_id\" value=\"$option_value\" /><br />";
    }
    
    /*
     * Callback function for overall offset setting field
     * @params: none
     * @return: none
     */
    public function draw_overall_offset_field(){
        $option_value = get_option($this->overall_offset_id);
        echo "<input type=\"text\" id=\"$this->overall_offset_id\" name=\"$this->overall_offset_id\" value=\"$option_value\" /><br />";
    }
    
    /*
     * Callback function for start Time setting field
     * @params: none
     * @return: none
     */
    public function draw_startTime_field(){
        $option_value = date("Y-m-d H:s", get_option($this->startTime_id));
        echo "<input type=\"text\" id=\"$this->startTime_id\" name=\"$this->startTime_id\" value=\"$option_value\" /><br />";
    }
    
    /*
     * Callback function for startTime Format field
     * @params: none
     * @return: none
     */
    public function draw_startTimeFormat_field(){
        $option_value = get_option($this->startTimeFormat_id);
        echo "<input type=\"text\" id=\"$this->startTimeFormat_id\" name=\"$this->startTimeFormat_id\" value=\"$option_value\" /><br />";
    }
    
    /*
     * Callback function for disconnect time field
     * @params: none
     * @return: none
     */
    public function draw_disconnectTime_field(){
        $option_value = get_option($this->disconnectTime_id);
        echo "<input type=\"text\" id=\"$this->disconnectTime_id\" name=\"$this->disconnectTime_id\" value=\"$option_value\" /><br />";
    }
    
    /*
     * Callback function for countBots field
     * @params: none
     * @return: none
     */
    public function draw_countBots_field(){
        $option_value = get_option($this->countBots_id);
        if ($option_value)
            $check_value = "checked";
        else
            $check_value = "";
        echo "<input type=\"checkbox\" id=\"$this->countBots_id\" name=\"$this->countBots_id\" value=\"true\" checked=\"$check_value\"/><br />";
    }
    
    /*
     * Method which prepare offset value 
     * @params: $inputData
     * @return: $new_data
     */
    public function prepare_offset_value($inputData){
        if ($inputData == '')
            $new_data = '0';
        else
            $new_data = (int)$inputData;
        return $new_data;
    }
    
    /*
     * Method which prepare unix time stamp data before save in db
     * @params: $inputData
     * @return: $new_date
     */
    public function prepare_new_startTime($inputData){
        if (!$inputData)
            $new_date = time();
        else {
            $inputData = explode(" ", $inputData);
            $day = explode("-",$inputData[0]);
            $time = explode(":",$inputData[1]);
            $new_date = mktime($time[0], $time[1], 0, $day[1], $day[2], $day[0]);
        }
        return $new_date;
    }
    
    /*
     * Method which prepare time format value
     * @params: $inputData
     * @return: $new_data
     */
    public function prepare_startTimeFormat_value($inputData){
        if (!$inputData)
            $new_data = 'Y-m-d H:s';
        else
            $new_data = $inputData;
        return $new_data;
    }
    
    /*
     * Method which prepare disconnect time value 
     * @params: $inputData
     * @return: $new_data
     */
    public function prepare_disconnectTime_value($inputData){
        if ($inputData == '')
            $new_data = 300;
        else
            $new_data = (int)$inputData;
        return $new_data;
    }
    
    /*
     * Method which prepare countBots Flag value
     * @params: $inputData
     * @return: $new_data
     */
    public function prepare_countBots_value($inputData){
        if (!$inputData)
            $new_data = 0;
        else
            $new_data = 1;
        return $new_data;
    }


    /*
     * Method which get options values from db
     * @params: none
     * @return: none
     */
    private function get_option_values(){
        $this->online_offset_value = get_option($this->online_offset_id);
        $this->overall_offset_value = get_option($this->overall_offset_id);
        $this->startTime_value = get_option($this->startTime_id);
        $this->startTimeFormat_value = get_option($this->startTimeFormat_id);
        $this->disconnectTime_value = get_option($this->disconnectTime_id);
        $this->countBots_value = get_option($this->countBots_id);
        
        if (!$this->online_offset_value) {
            add_option($this->online_offset_id, $this->online_offset_default_value, null, false);
            $this->online_offset_value = $this->online_offset_default_value;
        }
        
        if (!$this->overall_offset_value) {
            add_option($this->overall_offset_id, $this->overall_offset_default_value, null, false);
            $this->overall_offset_value = $this->overall_offset_default_value;
        }
        
        if (!$this->startTime_value) {
            add_option($this->startTime_id,$this->startTime_default_value, null, false);
            $this->startTime_value = $this->startTime_default_value;
        }
        
        if (!$this->startTimeFormat_value) {
            add_option($this->startTimeFormat_id, $this->startTimeFormat_default_value, null, false);
            $this->startTimeFormat_value = $this->startTimeFormat_default_value;
        }
        
        if (!$this->disconnectTime_value) {
            add_option($this->disconnectTime_id, $this->disconnectTime_default_value, null, false);
            $this->disconnectTime_value = $this->disconnectTime_default_value;
        }
        
        if (!$this->countBots_value) {
            add_option($this->countBots_id, $this->countBots_default_value, null, false);
            $this->countBots_value = $this->countBots_default_value;
        }
    }
    
    /*
     * Method which check is user already online on page
     * @params: $userIP
     * @return: true or false
     */
    private function check_user_online($userIP){
        global $wpdb;
        $check = $wpdb->get_row("SELECT time FROM $this->online_counter_table WHERE IP = '$userIP'");
        if ($check)
            $result = true;
        else
            $result = false;
        return $result;
    }
    
    /*
     * Main method which counts users actuall online and overall users which visits page
     * @args: none
     * @return: none;
     */
    public function vc_count_users(){
        global $wpdb;
        $this->get_option_values();
        $IP = $_SERVER['REMOTE_ADDR'];
        $current_time = time();
        $checking_time_range = $current_time - $this->disconnectTime_value;
        
        $this->removeOldUsers($checking_time_range);
        
        $user_already_here = $this->check_user_online($IP);
        if ($user_already_here){
            $wpdb->update($this->online_counter_table, array('time'=>$current_time), array('IP' => $IP));
        } else { 
            if (!$this->countBots_value){ 
                if($this->isBot() == False){
                    $wpdb->insert($this->online_counter_table, array('IP'=>$IP, 'time'=>$current_time));
                    $this->increase_overall();
                }
            } else {
                $wpdb->insert($this->online_counter_table, array('IP'=>$IP, 'time'=>$current_time));
                $this->increase_overall();
            }
        }
        
        $online_users = $wpdb->get_results("SELECT * FROM $this->online_counter_table");
        $overall_count = $this->get_overall_count();
        $result = array('online_users' => sizeof($online_users)+$this->online_offset_value, 'overall_counter' => $overall_count+$this->overall_offset_value, 'counterStartTime' => date($this->startTimeFormat_value, $this->startTime_value));
        return $result;
    }
    
    
    /*
     * Function which remove old entries:
     * @params: $timeRange
     * @return: none
     */
    private function removeOldUsers($timeRange){
        global $wpdb;
        $wpdb->query("DELETE FROM $this->online_counter_table WHERE time < $timeRange");
    }
    
    /*
     * Function which increase overall counter (+1)
     * @params: none
     * @return: none
     */
    private function increase_overall(){
        global $wpdb;
        $wpdb->query("UPDATE $this->overall_counter_table SET visits = visits+1 WHERE id = 1");
    }
    
    /*
     * Function which gets overall count value
     * @params: none
     * @return: overall count value
     */
    private function get_overall_count(){
        global $wpdb;
        $count = $wpdb->get_row("SELECT visits FROM $this->overall_counter_table WHERE id = 1");
        if (!$count){
            $wpdb->insert($this->overall_counter_table, array('id'=>1, 'visits'=>0));
            $result = 0;
        } else {
            $result = $count->visits;
        }
        return $result;
    }
    
        
    /*
     * Method which check is it Bot or "normal" user on website
     * @params: none
     * @return: true or false
     */
    private function isBot(){
        include_once 'visits_counter_uaList.php';
        $ua = $_SERVER['HTTP_USER_AGENT'];
        foreach($uaList as $agent){
            if (strstr($ua, $agent)){
                return True;
            }
        }
        return False;
    }
}